/*	**************************************************************************************
    *									overlay.js
    *									==========
    *
    *	Author:	Geva Zeichner
	*	Forum:	http://forums.mozillazine.org/viewtopic.php?t=408043
    *	Date:	15/04/2006
    *	Edited:	25/05/2006
    **************************************************************************************/

window.addEventListener("load", function(e){IMDb_Preview.handle_load(e);}, false);

var IMDb_Preview =
{
	/*************************************************************************************
	 *	Handle_Load(e)
	 *	==============
	 *	Listeners, defs, inits...
	 *************************************************************************************/
	handle_load: function(e)
	{
		// Shhh... listen...
		window.addEventListener("mouseover", function(e){IMDb_Preview.handle_mouse_over(e);}, false);
		window.addEventListener("mousemove", function(e){IMDb_Preview.handle_mouse_move(e);}, false);
		window.addEventListener("mousedown", function(e){IMDb_Preview.terminate_preview();}, false);
		
		// Consts
		this.TITLE_REGEXP = /^((http:\/\/)(\w*\.)?((imdb\.com)(.*)(\/title\/tt\d{7})(\/)?))$/;
		this.ACTOR_REGEXP = /^((http:\/\/)(\w*\.)?((imdb\.com)(.*)(\/name\/nm\d{7})(\/)?))$/;
		this.MOVIE_TYPE = 1;
		this.ACTOR_TYPE = 2;
		this.NOT_SUBMITTED_IMG_PATH = "chrome://imdb_preview/skin/not_submitted.png";
		this.LOADING_IMG_PATH = "chrome://imdb_preview/skin/loading.gif";
		this.E404_IMG_PATH = "chrome://imdb_preview/skin/404.png";
		
		// Set variables
		this.prefs = Components.classes['@mozilla.org/preferences-service;1'].
						getService(Components.interfaces.nsIPrefService).
						getBranch('extensions.imdb_preview.');
		this.move_timer = "";
		this.mouse_over_url_target = "";
		this.loaded_url = " ";
		this.last_owner_doc = "";
		this.http_req = new XMLHttpRequest();
		this.is_showing = false;
		this.page_x = 0;
		this.page_y = 0;
		this.target_start_time = 0;
		this.poster_start_time = 0;
		this.poster_end_time = 0;
		
		// Init preview settings //
		
		this.poster_img = window._content.document.createElement("img");
		this.poster_img.addEventListener("load", function(e){IMDb_Preview.handle_img_load(e);}, false);
		this.poster_img.src = "about:blank";
		
		this.wrap_div = window._content.document.createElement("div");
		this.wrap_div.appendChild(this.poster_img);
		
		// Poster style settings
		this.wrap_div.style.visibility = "hidden";
		this.wrap_div.style.textTop = "true";
		this.wrap_div.style.zIndex = "1000";
		this.wrap_div.style.position = "absolute";
		this.wrap_div.style.left = "0px";
		this.wrap_div.style.top = "0px";
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Over(e)
	 *	====================
	 *	Gets info about what we stepped into
	 *************************************************************************************/
	handle_mouse_over: function(e)
	{
		this.page_x = e.pageX;
		this.page_y = e.pageY;
		this.mouse_over_url_target = unescape(e.target.href);
		this.last_owner_doc = e.originalTarget.ownerDocument;
		
		this.terminate_preview();
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Move(e)
	 *	====================
	 *	Checks if we are halting somewhere
	 *************************************************************************************/
	handle_mouse_move: function(e)
	{
		clearTimeout(this.move_timer);
		this.move_timer = setTimeout(function(){IMDb_Preview.handle_mouse_halt();},
									 this.prefs.getIntPref("halt_time"));
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Halt()
	 *	===================
	 *	Checks if we halted on the right spot
	 *************************************************************************************/
	handle_mouse_halt: function()
	{
		// If standing on url that is different from the one stood on before
		if (this.mouse_over_url_target != this.loaded_url)
		{
			// If it's a movie link
			if (this.mouse_over_url_target.match(this.TITLE_REGEXP))
			{
				// Load url
				this.load_target(this.mouse_over_url_target, this.MOVIE_TYPE);
			}
			
			// If it's an actor link
			else if (this.mouse_over_url_target.match(this.ACTOR_REGEXP))
			{
				// Load url
				this.load_target(this.mouse_over_url_target, this.ACTOR_TYPE);
			}
		}
		this.loaded_url = this.mouse_over_url_target;
	},
	
	/*************************************************************************************
	 *	Load_Target(url_target, target_type)
	 *	====================================
	 *	Retrives the target's html code and triggers its processing
	 *************************************************************************************/
	load_target: function(url_target, target_type)
	{
		this.is_showing = true;
		this.target_start_time = new Date();
		this.dump("Loading...\n");
		
		// Show 'loading...' poster
		this.load_img(this.LOADING_IMG_PATH);
		
		// Load target
		this.http_req.open('GET', url_target, true);
		this.http_req.onreadystatechange = function()
		{
			// If loading completed
			if (IMDb_Preview.http_req.readyState == 4)
			IMDb_Preview.timer = setTimeout( function ()
			{
				try
				{
					// With no error
					if (IMDb_Preview.http_req.status == 200)
						IMDb_Preview.process_html(IMDb_Preview.http_req.responseText, target_type);
					else
					{
						IMDb_Preview.load_img(IMDb_Preview.E404_IMG_PATH);
						IMDb_Preview.dump("Error loading page\n");
					}
				}
				catch(err)
				{
					IMDb_Preview.dump(err);
				}
			}, 0);
		};
		this.http_req.send(null);
	},
	
	/*************************************************************************************
	 *	Process_HTML(html_source, target_type)
	 *	======================================
	 *	Gets the page's html code.
	 *	The poster's img src is extracted from the html and then sent to show_preview_img.
	 *************************************************************************************/
	process_html: function(html_source, target_type)
	{
		var img_src = -1;
		
		var title_start_pos = html_source.indexOf('<title>') + 7;
		var title_end_pos = html_source.indexOf('</title>', title_start_pos);
		var title_name = html_source.substring(title_start_pos, title_end_pos);
		this.dump(title_name + ":\n");
		
		// If the target it a movie page
		if (target_type == this.MOVIE_TYPE)
		{
			img_src = this.process_movie_html(html_source);
		}
		// If the target it an actor page
		else if (target_type == this.ACTOR_TYPE)
		{
			img_src = this.process_actor_html(html_source);
		}
		
		if (img_src == -1)
		{
			img_src = this.NOT_SUBMITTED_IMG_PATH;
			this.dump("Poster Not Submitted\n\n");
		}
		else
			this.dump(img_src + "\n\n");
		
		// Load poster
		this.load_img(img_src);
	},
	
	/*************************************************************************************
	 *	Process_Movie_HTML(html_source)
	 *	===============================
	 *************************************************************************************/
	process_movie_html: function(html_source)
	{
		var img_src = -1;
		var poster_pos = html_source.indexOf('<a name="poster"');
		
		if (poster_pos != -1)
		{
			var img_src_start_pos = html_source.indexOf('src="', poster_pos) + 5;
			var img_src_end_pos = html_source.indexOf('" height', img_src_start_pos);
			img_src = html_source.substring(img_src_start_pos, img_src_end_pos);
		}
		
		return(img_src);
	},
	
	/*************************************************************************************
	 *	Process_Actor_HTML(html_source)
	 *	===============================
	 *************************************************************************************/
	process_actor_html: function(html_source)
	{
		var img_src = -1;
		var poster_pos = html_source.indexOf('<a name="headshot"');
		
		if (poster_pos != -1)
		{
			var img_src_start_pos = html_source.indexOf('src="', poster_pos) + 5;
			var img_src_end_pos = html_source.indexOf('" width', img_src_start_pos);
			img_src = html_source.substring(img_src_start_pos, img_src_end_pos);
		}
		
		return(img_src);
	},
	
	/*************************************************************************************
	 *	Load_IMG(img_src)
	 *	=================
	 *	Loads an image into the img object
	 *************************************************************************************/
	load_img: function(img_src)
	{
		if (this.is_showing == true)
		{
			this.poster_img.src = img_src;
			this.last_owner_doc.body.appendChild(this.wrap_div);
			
			this.poster_start_time = new Date();
		}
	},
	
	/*************************************************************************************
	 *	Handle_IMG_Load(e)
	 *	==================
	 *	When the img finshed to load
	 *************************************************************************************/
	handle_img_load: function(e)
	{
		if (this.poster_img.src != "about:blank")
		{
			this.wrap_div.style.visibility = "hidden";
			this.show_preview();
			
			this.poster_end_time = new Date();
			
			if ((this.poster_img.src != this.NOT_SUBMITTED_IMG_PATH) &&
				(this.poster_img.src != this.LOADING_IMG_PATH) &&
				(this.poster_img.src != this.E404_IMG_PATH))
			{
				var curr_date = new Date();
				this.dump("Target loading took: " + (this.poster_start_time -
													 this.target_start_time) + "ms\n" +
						  "Poster loading took: " + (this.poster_end_time -
													 this.poster_start_time) + "ms\n" +
						  "Total loading took: " + (this.poster_end_time -
													 this.target_start_time) + "ms\n\n");
			}
		}
	},
	
	/*************************************************************************************
	 *	Show_Preview()
	 *	==============
	 *	Positions the preview and reveals it
	 *************************************************************************************/
	show_preview: function()
	{
		var doc = this.last_owner_doc;
		var top, left = 0;
		
		// If in upper half of window
		if (this.page_y < (doc.defaultView.innerHeight / 2) + doc.defaultView.pageYOffset)
			
			// Display preview below cursor
			top = this.page_y + 32;
		else
			top = this.page_y - this.poster_img.height - 10;
		
		// If in the left half of window
		if (this.page_x < (doc.defaultView.innerWidth / 2) + doc.defaultView.pageXOffset)
			
			// Display preview to the right of the cursor
			left = this.page_x;
		else
			left = this.page_x - this.poster_img.width;
		
		this.wrap_div.style.top = top + "px";
		this.wrap_div.style.left = left + "px";
		
		// Reveals the preview
		this.wrap_div.style.visibility = "visible";
	},
	
	/*************************************************************************************
	 *	Terminate_Preview()
	 *	===================
	 *	I'll be back...
	 *************************************************************************************/
	terminate_preview: function()
	{
		if (this.is_showing == true)
		{
			this.is_showing = false;
			this.poster_img.src = "about:blank";
			this.loaded_url = " ";
			this.wrap_div.style.visibility = "hidden";
			this.http_req.abort();
		}
	},
	
	/*************************************************************************************
	 *	Dump(text)
	 *	==========
	 *************************************************************************************/
	dump: function(text)
	{
		if (this.prefs.getBoolPref("dump") == true)
		{
			dump(text);
		}
	}
};